@@ -5,13 +5,14 @@
 */
 
 #include "plasmadesktoptheme.h"
-#include <QQmlEngine>
-#include <QQmlContext>
+#include <KIconLoader>
+#include <QDebug>
 #include <QGuiApplication>
 #include <QPalette>
-#include <QDebug>
+#include <QQmlContext>
+#include <QQmlEngine>
+#include <QQuickRenderControl>
 #include <QQuickWindow>
-#include <KIconLoader>
 
 #include <KColorScheme>
 #include <KConfigGroup>
@@ -43,16 +44,27 @@ public:
         , buttonScheme(QPalette::Active, KColorScheme::ColorSet::Button)
         , viewScheme(QPalette::Active, KColorScheme::ColorSet::View)
     {
-        connect(qGuiApp, &QGuiApplication::paletteChanged,
-                this, &StyleSingleton::refresh);
+        connect(qGuiApp, &QGuiApplication::paletteChanged, this, &StyleSingleton::refresh);
 
         // Use DBus in order to listen for kdeglobals changes directly, as the
         // QApplication doesn't expose the font variants we're looking for,
         // namely smallFont.
-        QDBusConnection::sessionBus().connect( QString(),
-        QStringLiteral( "/KGlobalSettings" ),
-        QStringLiteral( "org.kde.KGlobalSettings" ),
-        QStringLiteral( "notifyChange" ), this, SIGNAL(configurationChanged()));
+        QDBusConnection::sessionBus().connect(QString(),
+                                              QStringLiteral("/KGlobalSettings"),
+                                              QStringLiteral("org.kde.KGlobalSettings"),
+                                              QStringLiteral("notifyChange"),
+                                              this,
+                                              SLOT(notifyWatchersConfigurationChange()));
+
+        connect(qGuiApp, &QGuiApplication::fontDatabaseChanged, this, &StyleSingleton::notifyWatchersConfigurationChange);
+
+        // Use NativeTextRendering as the default text rendering type when the scale factor is an integer.
+        // NativeTextRendering is still distorted sometimes with fractional scale factors,
+        // despite https://bugreports.qt.io/browse/QTBUG-67007 being closed.
+        qreal devicePixelRatio = qGuiApp->devicePixelRatio();
+        QQuickWindow::TextRenderType defaultTextRenderType =
+            int(devicePixelRatio) == devicePixelRatio ? QQuickWindow::NativeTextRendering : QQuickWindow::QtTextRendering;
+        QQuickWindow::setTextRenderType(defaultTextRenderType);
     }
 
     void refresh()
@@ -61,7 +73,7 @@ public:
         buttonScheme = KColorScheme(QPalette::Active, KColorScheme::ColorSet::Button);
         viewScheme = KColorScheme(QPalette::Active, KColorScheme::ColorSet::View);
 
-        Q_EMIT paletteChanged();
+        notifyWatchersPaletteChange();
     }
 
     Colors loadColors(Kirigami::PlatformTheme::ColorSet cs, QPalette::ColorGroup group)
@@ -102,7 +114,7 @@ public:
         Colors ret = {{}, KColorScheme(group, KColorScheme::ColorSet::Selection), KColorScheme(group, set)};
 
         QPalette pal;
-        for (auto state : { QPalette::Active, QPalette::Inactive, QPalette::Disabled }) {
+        for (auto state : {QPalette::Active, QPalette::Inactive, QPalette::Disabled}) {
             pal.setBrush(state, QPalette::WindowText, ret.scheme.foreground());
             pal.setBrush(state, QPalette::Window, ret.scheme.background());
             pal.setBrush(state, QPalette::Base, ret.scheme.background());
@@ -129,12 +141,24 @@ public:
         return ret;
     }
 
+    void notifyWatchersPaletteChange()
+    {
+        for (auto watcher : qAsConst(watchers)) {
+            watcher->syncColors();
+        }
+    }
+
+    Q_SLOT void notifyWatchersConfigurationChange()
+    {
+        for (auto watcher : qAsConst(watchers)) {
+            watcher->syncFont();
+        }
+    }
+
     KColorScheme buttonScheme;
     KColorScheme viewScheme;
 
-Q_SIGNALS:
-    void configurationChanged();
-    void paletteChanged();
+    QVector<QPointer<PlasmaDesktopTheme>> watchers;
 
 private:
     QHash<QPair<Kirigami::PlatformTheme::ColorSet, QPalette::ColorGroup>, Colors> m_cache;
@@ -145,71 +169,75 @@ PlasmaDesktopTheme::PlasmaDesktopTheme(QObject *parent)
     : PlatformTheme(parent)
 {
     setSupportsIconColoring(true);
-    m_parentItem = qobject_cast<QQuickItem *>(parent);
 
-    //null in case parent is a normal QObject
-    if (m_parentItem) {
-        connect(m_parentItem.data(), &QQuickItem::enabledChanged,
-                this, &PlasmaDesktopTheme::syncColors);
-        if (m_parentItem && m_parentItem->window()) {
-            connect(m_parentItem->window(), &QWindow::activeChanged,
-                    this, &PlasmaDesktopTheme::syncColors);
-            m_window = m_parentItem->window();
-        }
-        connect(m_parentItem.data(), &QQuickItem::windowChanged,
-                this, [this]() {
-                    if (m_window) {
-                        disconnect(m_window.data(), &QWindow::activeChanged,
-                                this, &PlasmaDesktopTheme::syncColors);
-                    }
-                    if (m_parentItem && m_parentItem->window()) {
-                        connect(m_parentItem->window(), &QWindow::activeChanged,
-                                this, &PlasmaDesktopTheme::syncColors);
-                    }
-                    syncColors();
-                });
+    auto parentItem = qobject_cast<QQuickItem *>(parent);
+    if (parentItem) {
+        connect(parentItem, &QQuickItem::enabledChanged, this, &PlasmaDesktopTheme::syncColors);
+        connect(parentItem, &QQuickItem::windowChanged, this, &PlasmaDesktopTheme::syncWindow);
     }
 
-    //TODO: correct? depends from https://codereview.qt-project.org/206889
-    connect(qGuiApp, &QGuiApplication::fontDatabaseChanged, this, [this]() {setDefaultFont(qApp->font());});
-    configurationChanged();
-
-    connect(this, &PlasmaDesktopTheme::colorSetChanged,
-            this, &PlasmaDesktopTheme::syncColors);
-    connect(this, &PlasmaDesktopTheme::colorGroupChanged,
-            this, &PlasmaDesktopTheme::syncColors);
-
-    connect(s_style->data(), &StyleSingleton::paletteChanged,
-            this, &PlasmaDesktopTheme::syncColors);
-
-    connect(s_style->data(), &StyleSingleton::configurationChanged,
-            this, &PlasmaDesktopTheme::configurationChanged);
+    (*s_style)->watchers.append(this);
 
+    syncFont();
+    syncWindow();
     syncColors();
 }
 
-PlasmaDesktopTheme::~PlasmaDesktopTheme() = default;
+PlasmaDesktopTheme::~PlasmaDesktopTheme()
+{
+    (*s_style)->watchers.removeOne(this);
+}
 
-void PlasmaDesktopTheme::configurationChanged()
+void PlasmaDesktopTheme::syncWindow()
+{
+    if (m_window) {
+        disconnect(m_window.data(), &QWindow::activeChanged, this, &PlasmaDesktopTheme::syncColors);
+    }
+
+    QWindow *window = nullptr;
+
+    auto parentItem = qobject_cast<QQuickItem *>(parent());
+    if (parentItem) {
+        QQuickWindow *qw = parentItem->window();
+
+        window = QQuickRenderControl::renderWindowFor(qw);
+        if (!window) {
+            window = qw;
+        }
+        if (qw) {
+            connect(qw, &QQuickWindow::sceneGraphInitialized, this, &PlasmaDesktopTheme::syncWindow);
+        }
+    }
+    m_window = window;
+
+    if (window) {
+        connect(m_window.data(), &QWindow::activeChanged, this, &PlasmaDesktopTheme::syncColors);
+        syncColors();
+    }
+}
+
+void PlasmaDesktopTheme::syncFont()
 {
     KSharedConfigPtr ptr = KSharedConfig::openConfig();
-    KConfigGroup general( ptr->group("general") );
+    KConfigGroup general(ptr->group("general"));
     setSmallFont(general.readEntry("smallestReadableFont", []() {
         auto smallFont = qApp->font();
         if (smallFont.pixelSize() != -1) {
-            smallFont.setPixelSize(smallFont.pixelSize()-2);
+            smallFont.setPixelSize(smallFont.pixelSize() - 2);
         } else {
-            smallFont.setPointSize(smallFont.pointSize()-2);
+            smallFont.setPointSize(smallFont.pointSize() - 2);
         }
         return smallFont;
     }()));
+
+    setDefaultFont(qGuiApp->font());
 }
 
 QIcon PlasmaDesktopTheme::iconFromTheme(const QString &name, const QColor &customColor)
 {
     QPalette pal = palette();
     if (customColor != Qt::transparent) {
-        for (auto state : { QPalette::Active, QPalette::Inactive, QPalette::Disabled }) {
+        for (auto state : {QPalette::Active, QPalette::Inactive, QPalette::Disabled}) {
             pal.setBrush(state, QPalette::WindowText, customColor);
         }
     }
@@ -222,22 +250,22 @@ QIcon PlasmaDesktopTheme::iconFromTheme(const QString &name, const QColor &custo
 void PlasmaDesktopTheme::syncColors()
 {
     QPalette::ColorGroup group = (QPalette::ColorGroup)colorGroup();
-    if (m_parentItem) {
-        if (!m_parentItem->isEnabled()) {
+    auto parentItem = qobject_cast<QQuickItem *>(parent());
+    if (parentItem) {
+        if (!parentItem->isEnabled()) {
             group = QPalette::Disabled;
-        //Why also checking the window is exposed?
-        //in the case of QQuickWidget the window() will never be active
-        //and the widgets will always have the inactive palette.
-        // better to always show it active than always show it inactive
-        } else if (m_parentItem->window() && !m_parentItem->window()->isActive() && m_parentItem->window()->isExposed()) {
+            // Why also checking the window is exposed?
+            // in the case of QQuickWidget the window() will never be active
+            // and the widgets will always have the inactive palette.
+            // better to always show it active than always show it inactive
+        } else if (m_window && !m_window->isActive() && m_window->isExposed()) {
             group = QPalette::Inactive;
         }
     }
 
     const auto colors = (*s_style)->loadColors(colorSet(), group);
-    setPalette(colors.palette);
 
-    //foreground
+    // foreground
     setTextColor(colors.scheme.foreground(KColorScheme::NormalText).color());
     setDisabledTextColor(colors.scheme.foreground(KColorScheme::InactiveText).color());
     setHighlightedTextColor(colors.selectionScheme.foreground(KColorScheme::NormalText).color());
@@ -248,8 +276,7 @@ void PlasmaDesktopTheme::syncColors()
     setNeutralTextColor(colors.scheme.foreground(KColorScheme::NeutralText).color());
     setPositiveTextColor(colors.scheme.foreground(KColorScheme::PositiveText).color());
 
-
-    //background
+    // background
     setBackgroundColor(colors.scheme.background(KColorScheme::NormalBackground).color());
     setAlternateBackgroundColor(colors.scheme.background(KColorScheme::AlternateBackground).color());
     setHighlightColor(colors.selectionScheme.background(KColorScheme::NormalBackground).color());
@@ -260,71 +287,27 @@ void PlasmaDesktopTheme::syncColors()
     setNeutralBackgroundColor(colors.scheme.background(KColorScheme::NeutralBackground).color());
     setPositiveBackgroundColor(colors.scheme.background(KColorScheme::PositiveBackground).color());
 
-    //decoration
+    // decoration
     setHoverColor(colors.scheme.decoration(KColorScheme::HoverColor).color());
     setFocusColor(colors.scheme.decoration(KColorScheme::FocusColor).color());
-
-    //legacy stuff
-    m_buttonTextColor = (*s_style)->buttonScheme.foreground(KColorScheme::NormalText).color();
-    m_buttonBackgroundColor = (*s_style)->buttonScheme.background(KColorScheme::NormalBackground).color();
-    m_buttonHoverColor = (*s_style)->buttonScheme.decoration(KColorScheme::HoverColor).color();
-    m_buttonFocusColor = (*s_style)->buttonScheme.decoration(KColorScheme::FocusColor).color();
-
-    m_viewTextColor = (*s_style)->viewScheme.foreground(KColorScheme::NormalText).color();
-    m_viewBackgroundColor = (*s_style)->viewScheme.background(KColorScheme::NormalBackground).color();
-    m_viewHoverColor = (*s_style)->viewScheme.decoration(KColorScheme::HoverColor).color();
-    m_viewFocusColor = (*s_style)->viewScheme.decoration(KColorScheme::FocusColor).color();
-
-    emit colorsChanged();
 }
 
-QColor PlasmaDesktopTheme::buttonTextColor() const
+bool PlasmaDesktopTheme::event(QEvent *event)
 {
-    qWarning() << "WARNING: buttonTextColor is deprecated, use textColor with colorSet: Theme.Button instead";
-    return m_buttonTextColor;
-}
+    if (event->type() == Kirigami::PlatformThemeEvents::DataChangedEvent::type) {
+        syncFont();
+        syncColors();
+    }
 
-QColor PlasmaDesktopTheme::buttonBackgroundColor() const
-{
-    qWarning() << "WARNING: buttonBackgroundColor is deprecated, use backgroundColor with colorSet: Theme.Button instead";
-    return m_buttonBackgroundColor;
-}
+    if (event->type() == Kirigami::PlatformThemeEvents::ColorSetChangedEvent::type) {
+        syncColors();
+    }
 
-QColor PlasmaDesktopTheme::buttonHoverColor() const
-{
-    qWarning() << "WARNING: buttonHoverColor is deprecated, use backgroundColor with colorSet: Theme.Button instead";
-    return m_buttonHoverColor;
-}
+    if (event->type() == Kirigami::PlatformThemeEvents::ColorGroupChangedEvent::type) {
+        syncColors();
+    }
 
-QColor PlasmaDesktopTheme::buttonFocusColor() const
-{
-    qWarning() << "WARNING: buttonFocusColor is deprecated, use backgroundColor with colorSet: Theme.Button instead";
-    return m_buttonFocusColor;
-}
-
-
-QColor PlasmaDesktopTheme::viewTextColor() const
-{
-    qWarning()<<"WARNING: viewTextColor is deprecated, use backgroundColor with colorSet: Theme.View instead";
-    return m_viewTextColor;
-}
-
-QColor PlasmaDesktopTheme::viewBackgroundColor() const
-{
-    qWarning() << "WARNING: viewBackgroundColor is deprecated, use backgroundColor with colorSet: Theme.View instead";
-    return m_viewBackgroundColor;
-}
-
-QColor PlasmaDesktopTheme::viewHoverColor() const
-{
-    qWarning() << "WARNING: viewHoverColor is deprecated, use backgroundColor with colorSet: Theme.View instead";
-    return m_viewHoverColor;
-}
-
-QColor PlasmaDesktopTheme::viewFocusColor() const
-{
-    qWarning() << "WARNING: viewFocusColor is deprecated, use backgroundColor with colorSet: Theme.View instead";
-    return m_viewFocusColor;
+    return PlatformTheme::event(event);
 }
 
 #include "plasmadesktoptheme.moc"
